var Sequencing = function()
{
	var sequencing = {};
	var debug = 0;
	
	var runParameters;
	var sequencers;
	var endDateIsValid = false;

	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};

	// Page initialization
	sequencing.initPage = function()
	{
		// Step 1
		Events.addEventHandler('seqRuns', 'change', sequencing.sequencingRunOnChange);
		Events.addEventHandler('runParametersXml', 'change', sequencing.runParametersXmlFileOnChange);
		Buttons.addClickHandler('searchRunArchive', sequencing.searchRunArchive);
		Events.addEventHandler('endDate', 'blur', sequencing.endDateTimeOnChange);
		Events.addEventHandler('endTime', 'blur', sequencing.endDateTimeOnChange);

		Events.addEventHandler('step-1', 'wizard-validate', sequencing.validateStep1);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', sequencing.submit);
	
		// Load open grid clusters
		var frm = document.forms['reggie'];
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, sequencing.clustersLoaded);

		// Load sequencers
		var url = '../Hardware.servlet?ID='+App.getSessionId();
		url += '&cmd=GetHardware&subtype=SEQUENCER&annotations=SERIAL_NUMBER';
		Wizard.asyncJsonRequest(url, sequencing.sequencersLoaded);
		
		Wizard.showLoadingAnimation('Loading sequencing runs...');	
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=GetActiveSequencingRuns';
		Wizard.asyncJsonRequest(url, sequencing.initializeStep1);
	}

	sequencing.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
		}
		if (frm.clusters.length == 0)
		{
			frm.clusters[frm.clusters.length] = new Option('No available Open Grid clusters');
			frm.clusters.disabled = true;
			Doc.addClass('searchRunArchive', 'disabled');
		}
	}

	sequencing.sequencersLoaded = function(response)
	{
		sequencers = response.hardware;
	}
	
	sequencing.initializeStep1 = function(response)
	{
		var seqRuns = response.sequencingRuns;
		var frm = document.forms['reggie'];	

		if (seqRuns.length > 0)
		{
			for (var runNo=0; runNo < seqRuns.length; runNo++)
			{
				var seqRun = seqRuns[runNo];
				var option = sequencing.getOptionForSeqRun(seqRun);
				frm.seqRuns.options[frm.seqRuns.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No sequencing runs available for processing.');
			return;
		}
		
		Doc.show('step-1');
		Doc.show('gocancel');
		Doc.show('goregister');

		sequencing.sequencingRunOnChange();
		frm.seqRuns.focus();
	}
	
	sequencing.getOptionForSeqRun = function(seqRun)
	{
		var flowCell = seqRun.flowCell;
		var name = seqRun.name+': ';
		var numPools = flowCell.pools.length;
		var firstPoolNum = sequencing.getPoolNum(flowCell.pools[0].name);
		var lastPoolNum = sequencing.getPoolNum(flowCell.pools[flowCell.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += flowCell.pools[0].name + ' — ' + flowCell.pools[flowCell.pools.length-1].name;
			}
			else
			{
				name += flowCell.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += flowCell.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}

		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = flowCell.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}
		
		if (flowCell.FlowCellType)
		{
			name += ' [' + Strings.encodeTags(flowCell.FlowCellType) + ']';
		}
		
		if (seqRun.autoAnalyze)
		{
			name += ' [auto-analyze]';
		}

		var option = new Option(name, seqRun.id);
		option.title = title;
		option.seqRun = seqRun;	
		return option;
	}

		
	sequencing.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;
	}


	sequencing.sequencingRunOnChange = function()
	{
		sequencing.resetSeqRunInfo();
	}
	
	sequencing.resetSeqRunInfo = function()
	{
		var frm = document.forms['reggie'];	
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		
		if (seqRun.autoAnalyze)
		{
			var aa = seqRun.autoAnalyze;
			var img = JOB_IMAGE[aa.status] || 'info.png';
			var msg = '<span id="autoAnalyzeJob" class="link" data-item-type="JOB" data-item-id="'+aa.id+'"><img src="../images/'+img+'" style="margin-right: 0.5em;"></span>';

			msg += '<span class="itemsubtype">['+aa.status+']</span> ';
			if (aa.status == 'EXECUTING')
			{
				msg += aa.progress + '% ';
			}
			msg += Strings.encodeTags(seqRun.autoAnalyze.message);
			Wizard.setInputStatus('autoAnalyze', 'warning', 'Auto-analyze has been selected for this sequencing run. Manual registration in this step will stop the automatic processing.');
			Doc.element('autoAnalyze').innerHTML = msg;
			Doc.show('autoAnalyzeSection');
			Events.addEventHandler('autoAnalyzeJob', 'click', Items.itemOnClick);
			Wizard.showGoNextConfirmation(true, 'Check to verify registration of an auto-analyze sequencing run.');
		}
		else
		{
			Doc.hide('autoAnalyzeSection');
			Wizard.hideGoNextConfirmation();
		}

		// Flow cell info
		var fc = seqRun.flowCell;
		Doc.element('externalId').innerHTML = Strings.encodeTags(fc.FlowCellID || '');
		Doc.element('flowCellType').innerHTML = Strings.encodeTags(fc.FlowCellType || '');
		var pools = fc.pools;
		var html = '';
		var libPlateHtml = '';
		for (var poolNo = 0; poolNo < pools.length; poolNo++)
		{
			var pool = pools[poolNo];
			if (poolNo > 0) html += ', ';
			html += Strings.encodeTags(pool.name);
			if (pool.LibPlate && libPlateHtml.indexOf(pool.LibPlate.name) == -1)
			{
				if (libPlateHtml.length > 0) libPlateHtml += ', ';
				libPlateHtml += Strings.encodeTags(pool.LibPlate.name);
			}
		}
		Doc.element('pools').innerHTML = html;
		Doc.element('libPlate').innerHTML = libPlateHtml || 'n/a';

		// Sequencing info
		var sequencer = seqRun.sequencer;
		Doc.element('startDate').innerHTML = Reggie.reformatDate(seqRun.SequencingStart);
		Doc.element('position').innerHTML = Strings.encodeTags(seqRun.HiSeqPosition);
		Doc.element('sequencingCycles').innerHTML = Strings.encodeTags(fc.SequencingCycles);
		Doc.element('sequencer').innerHTML = sequencer ? 
				Strings.encodeTags(sequencer.name + ' ('+sequencer.SerialNumber+')') : '';
		frm.comments.value = seqRun.comments;
		
		// Loaded from RunParameters.xml
		Doc.element('experimentName').innerHTML = '';
		Doc.element('runId').innerHTML = '';
		Doc.element('runNumber').innerHTML = '';
		Doc.element('dataFolder').innerHTML = '';
		Doc.element('endDate').value = '';
		Doc.element('endTime').value = '';
		
		runParameters = null;
		Wizard.setInputStatus('runParametersXml');
		Wizard.setInputStatus('externalId');
		Wizard.setInputStatus('startDate');
		Wizard.setInputStatus('sequencingCycles');
		Wizard.setInputStatus('sequencer');
		Wizard.setInputStatus('position');
	}

	sequencing.runParametersXmlFileOnChange = function(event)
	{
		sequencing.resetSeqRunInfo();
		Wizard.setNoConfirmOnFirstStep(false);
		var frm = document.forms['reggie'];
		var file = event.target.files[0];
		
		var reader = new FileReader();
		reader.onload = sequencing.runParametersXmlFileLoaded;
		reader.readAsText(file);
	}
	
	sequencing.runParametersXmlFileLoaded = function(event)
	{
		runParameters = sequencing.parseRunParametersXmlFile(event.target.result);
		sequencing.validateAndCopyRunParameters(runParameters);
	}
	
	sequencing.searchRunArchive = function()
	{
		sequencing.resetSeqRunInfo();
		
		var frm = document.forms['reggie'];
		var flowCell = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun.flowCell;
		
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=SearchRunArchive';
		url += '&FlowCellID='+encodeURIComponent(flowCell.FlowCellID);
		url += '&cluster='+encodeURIComponent(frm.clusters.value);
		
		Wizard.setInputStatus('runParametersXml', 'checking', 'Searching...');
		Wizard.asyncJsonRequest(url, sequencing.runArchiveSearched);
	}

	sequencing.runArchiveSearched = function(response)
	{
		Wizard.setInputStatus('runParametersXml');
		if (response.exitStatus != 0)
		{
			Wizard.setInputStatus('runParametersXml', 'invalid', Strings.encodeTags(response.stderr).replace(/\n/g, '<br>'));
			return;
		}
		Wizard.setNoConfirmOnFirstStep(false);
		
		Doc.element('dataFolder').innerHTML = Strings.encodeTags(response.dataFolder);
		if (response.endDateTime)
		{
			Doc.element('endDate').value = response.endDateTime.substr(0, 8);
			Doc.element('endTime').value = response.endDateTime.substr(9, 4);
		}
		
		runParameters = sequencing.parseRunParametersXmlFile(response.runParameters);
		sequencing.validateAndCopyRunParameters(runParameters);
	}

	sequencing.parseRunParametersXmlFile = function(data)
	{
		var doc;
		var info = {};
		try
		{
			var parser = new DOMParser();
			doc = parser.parseFromString(data, "application/xml");
		
			if (doc.documentElement.tagName == 'parsererror')
			{
				throw new Error(doc.documentElement.firstChild.textContent.split(/\n/)[0]);
			}
			
			info.ExperimentName = sequencing.getTextValue(doc, 'ExperimentName');
			info.RunID = sequencing.getTextValue(doc, 'RunID');
			info.HiSeqPosition = 'A';
			info.ScannerID = sequencing.getTextValue(doc, 'InstrumentID');
			info.ScanNumber = parseInt(sequencing.getTextValue(doc, 'RunNumber'));
			info.FlowCellID = sequencing.getTextValue(doc, 'FlowCellSerial');
			info.SequencingCycles = sequencing.getTextValue(doc, 'Read1') + '-' + sequencing.getTextValue(doc, 'Index1Read') + '-' + sequencing.getTextValue(doc, 'Read2');
			info.SequencingStart = '20'+sequencing.getTextValue(doc, 'RunStartDate');
		}
		catch (err)
		{
			Wizard.setInputStatus('runParametersXml', 'invalid', err.toString());
			info.error = true;
		}
		
		if (debug) App.debug(JSON.stringify(info));
		return info;
	}

	sequencing.getTextValue = function(doc, selector)
	{
		var el = doc.querySelector(selector);
		if (el == null)
		{
			throw new Error("Can't find tag &lt;"+selector+"&gt;");
		}
		return el.textContent;
	}

	sequencing.validateAndCopyRunParameters = function(info)
	{
		if (info.error) return;
		var frm = document.forms['reggie'];
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		var fc = seqRun.flowCell;

		// Flow cell info
		Doc.element('externalId').innerHTML = info.FlowCellID || '';
		if (fc.FlowCellID != info.FlowCellID)
		{
			if (fc.FlowCellID.toLowerCase() != info.FlowCellID.toLowerCase())
			{
				Wizard.setInputStatus('externalId', 'invalid', 'Not same as registered barcode ('+Strings.encodeTags(fc.FlowCellID)+').');
				info.error = true;
			}
			else
			{
				Wizard.setInputStatus('externalId', 'warning', 'Not same as registered barcode ('+Strings.encodeTags(fc.FlowCellID)+').');
			}
		}

		// Seq run info
		Doc.element('startDate').innerHTML = Reggie.reformatDate(info.SequencingStart) || '';
		if (seqRun.SequencingStart != info.SequencingStart)
		{
			Wizard.setInputStatus('startDate', 'warning', 'Not same as registered start date ('+Reggie.reformatDate(seqRun.SequencingStart)+').');
		}
		Doc.element('sequencingCycles').innerHTML = info.SequencingCycles || '';
		if (fc.SequencingCycles != info.SequencingCycles)
		{
			Wizard.setInputStatus('sequencingCycles', 'warning', 'Not same as planned SequencingCycles ('+Strings.encodeTags(fc.SequencingCycles)+').');
		}
		Doc.element('position').innerHTML = info.HiSeqPosition || '';
		if (seqRun.HiSeqPosition != info.HiSeqPosition)
		{
			Wizard.setInputStatus('position', 'warning', 'Not same as planned position ('+Strings.encodeTags(seqRun.HiSeqPosition)+').');
		}

		if (info.ScannerID && seqRun.sequencer)
		{
			if (seqRun.sequencer.SerialNumber != info.ScannerID)
			{
				Doc.element('sequencer').innerHTML = info.ScannerID;
				Wizard.setInputStatus('sequencer', 'warning', 'Not same as registered sequencer ('+Strings.encodeTags(seqRun.sequencer.name)+').');
				// Try to find the sequencer
				if (sequencers && sequencers.length) 
				{
					for (var i = 0; i < sequencers.length; i++)
					{
						var seq = sequencers[i];
						if (seq.SerialNumber == info.ScannerID)
						{
							Doc.element('sequencer').innerHTML = Strings.encodeTags(seq.name + ' ('+seq.SerialNumber + ')');
							info.sequencer = seq;
						}
					}
				}
				
			}
		}
		
		
		Doc.element('experimentName').innerHTML = info.ExperimentName || '';
		Doc.element('runId').innerHTML = info.RunID || '';
		Doc.element('runNumber').innerHTML = info.ScanNumber || '';
		
		if (!info.error) Wizard.setInputStatus('runParametersXml', 'valid');
		frm.endDate.focus();
		
	}

	sequencing.validateStep1 = function(event)
	{
		if (!runParameters)
		{
			Wizard.setInputStatus('runParametersXml', 'invalid', 'Missing');
		}
		if (!runParameters || runParameters.error || !endDateIsValid) 
		{
			event.preventDefault();
		}
	}
	

	sequencing.endDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		endDateIsValid = false;
		Wizard.setInputStatus('endDate');
		
		var endDate = frm.endDate.value;
		var endTime = frm.endTime.value;
		
		if (endDate == '')
		{
			Wizard.setInputStatus('endDate', 'invalid', 'Missing date');
			return;
		}
		else
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			endDate = Reggie.autoFillDate(endDate);
			frm.endDate.value = endDate;

			if (!Dates.isDate(endDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('endDate', 'invalid', 'Not a valid date');
				return;
			}
		}
		
		if (endTime == '')
		{
			Wizard.setInputStatus('endDate', 'invalid', 'Missing time');
			return;
		}
		else
		{
			endTime = Reggie.autoFillTime(endTime);
			frm.endTime.value = endTime;
			
			if (!Dates.isDate(endDate + ' ' + endTime, 'yyyyMMdd HHmm'))
			{
				Wizard.setInputStatus('endDate', 'invalid', 'Not a valid time');
				return;
			}
		}


		Wizard.setInputStatus('endDate', 'valid');
		endDateIsValid = true;
	}

	
	sequencing.submit = function()
	{
		var frm = document.forms['reggie'];
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		seqRun.endDate = Strings.trim(frm.endDate.value + ' ' + frm.endTime.value);
		seqRun.comments = frm.comments.value;
			
		var submitInfo = {};
		submitInfo.sequencingRun = seqRun;
		submitInfo.runParameters = runParameters;
		
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterSequencingEnded';
			
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, sequencing.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	sequencing.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return sequencing;
}();

Doc.onLoad(Sequencing.initPage);

